@@ -3,119 +3,84 @@ use v5.10;
 use Dancer ':syntax';
 use Dancer::Plugin;
 use Scalar::Util 'blessed';
-use Time::HiRes 'time';
-
-our $VERSION = '1.09';
 
 no if $] >= 5.018, warnings => 'experimental::smartmatch';
 
-use constant PLUGIN_NAME => 'restrpc';
-
 use Dancer::RPCPlugin::CallbackResult;
 use Dancer::RPCPlugin::ErrorResponse;
 use Dancer::RPCPlugin::DispatchFromConfig;
 use Dancer::RPCPlugin::DispatchFromPod;
 use Dancer::RPCPlugin::DispatchItem;
 use Dancer::RPCPlugin::DispatchMethodList;
-use Dancer::RPCPlugin::FlattenData;
 
 my %dispatch_builder_map = (
     pod    => \&build_dispatcher_from_pod,
     config => \&build_dispatcher_from_config,
 );
 
-register PLUGIN_NAME ,=> sub {
+register restrpc => sub {
     my($self, $base_url, $arguments) = plugin_args(@_);
 
     my $publisher;
-    GIVEN: {
-        local $_ = $arguments->{publish} // 'config';
-        exists($dispatch_builder_map{$_}) && do {
+    given ($arguments->{publish} // 'config') {
+        when (exists $dispatch_builder_map{$_}) {
             $publisher = $dispatch_builder_map{$_};
+
             $arguments->{arguments} = plugin_setting() if $_ eq 'config';
-            last GIVEN;
-        };
-        do {
+        }
+        default {
             $publisher = $_;
-        };
+        }
     }
+
+    my $code_wrapper = $arguments->{code_wrapper} // sub {
+        my $code = shift;
+        my $pkg  = shift;
+        $code->(@_);
+    };
+    my $callback = $arguments->{callback};
     my $dispatcher = $publisher->($arguments->{arguments}, $base_url);
 
     my $lister = Dancer::RPCPlugin::DispatchMethodList->new();
     $lister->set_partial(
-        protocol => PLUGIN_NAME,
+        protocol => 'restrpc',
         endpoint => $base_url,
         methods  => [ sort keys %{ $dispatcher } ],
     );
 
-    my $code_wrapper = $arguments->{code_wrapper}
-        ? $arguments->{code_wrapper}
-        : sub {
-            my $code = shift;
-            my $pkg  = shift;
-            $code->(@_);
-        };
-    my $callback = $arguments->{callback};
-
-    debug("Starting restrpc-handler build: ", $lister);
     my $handle_call = sub {
-        my ($ct) = (split /;\s*/, request->content_type, 2);
-        if ($ct ne 'application/json') {
+        if (request->content_type ne 'application/json') {
             pass();
         }
-        debug("[handle_restrpc_request] Processing: ", request->body);
 
         # method_name should exist
         my ($method_name) = request->path =~ m{$base_url/(\w+)};
         if (! exists $dispatcher->{$method_name}) {
-            warning("$base_url/#$method_name not found, pass()");
             pass();
         }
 
         content_type 'application/json';
-        my $response;
         my $method_args = request->body
             ? from_json(request->body)
             : undef;
-
-        debug("[handle_restrpc_call($method_name)] ", $method_args);
-        my $start_request = time();
         my Dancer::RPCPlugin::CallbackResult $continue = eval {
-            local $Dancer::RPCPlugin::ROUTE_INFO = {
-                plugin        => PLUGIN_NAME,
-                endpoint      => $base_url,
-                rpc_method    => $method_name,
-                full_path     => request->path,
-                http_method   => uc(request->method),
-            };
             $callback
                 ? $callback->(request(), $method_name, $method_args)
                 : callback_success();
         };
 
+        my $response;
         if (my $error = $@) {
-            my $error_response = Dancer::RPCPlugin::ErrorResponse->new(
+            $response = Dancer::RPCPlugin::ErrorResponse->new(
                 error_code => 500,
                 error_message => $error,
-            );
-            status $error_response->return_status(PLUGIN_NAME);
-            $response = $error_response->as_restrpc_error;
+            )->as_restrpc_error;
         }
-        elsif (!blessed($continue) || !$continue->isa('Dancer::RPCPlugin::CallbackResult')) {
-            my $error_response = Dancer::RPCPlugin::ErrorResponse->new(
-                error_code    => 500,
-                error_message => "Internal error: 'callback_result' wrong class " . blessed($continue),
-            );
-            status $error_response->return_status(PLUGIN_NAME);
-            $response = $error_response->as_restrpc_error;
-        }
-        elsif (blessed($continue) && !$continue->success) {
-            my $error_response = Dancer::RPCPlugin::ErrorResponse->new(
+        elsif (! $continue->success) {
+            $response = Dancer::RPCPlugin::ErrorResponse->new(
                 error_code    => $continue->error_code,
                 error_message => $continue->error_message,
-            );
-            status $error_response->return_status(PLUGIN_NAME);
-            $response = $error_response->as_restrpc_error;
+            )->as_restrpc_error;
         }
         else {
             my Dancer::RPCPlugin::DispatchItem $di = $dispatcher->{$method_name};
@@ -126,38 +91,20 @@ register PLUGIN_NAME ,=> sub {
                 $code_wrapper->($handler, $package, $method_name, $method_args);
             };
 
-            debug("[handled_restrpc_request($method_name)] ", flatten_data($response));
             if (my $error = $@) {
-                my $error_response = blessed($error) && $error->can('as_restrpc_error')
-                    ? $error
-                    : error_response(
-                            error_code    => -32500,
-                            error_message => $error,
-                            error_data    => $method_args,
-                    );
-                status $error_response->return_status(PLUGIN_NAME);
-                $response = $error_response->as_restrpc_error;
+                $response = Dancer::RPCPlugin::ErrorResponse->new(
+                    error_code => 500,
+                    error_message => $error,
+                )->as_restrpc_error;
             }
-            elsif (blessed($response) && $response->can('as_restrpc_error')) {
+            if (blessed($response) && $response->can('as_restrpc_error')) {
                $response = $response->as_restrpc_error;
             }
-            elsif (blessed($response)) {
-                $response = flatten_data($response);
-            }
         }
         $response = { result => $response } if !ref($response);
-        my $jsonise_options = {canonical => 1};
-        if (config->{encoding} && config->{encoding} =~ m{^utf-?8$}i) {
-            $jsonise_options->{utf8} = 1;
-        }
-        info( sprintf(
-            "[RPC::RESTRPC] request for %s took %.4fs",
-            $method_name, time() - $start_request
-        ));
-        return to_json($response, $jsonise_options);
+        return to_json($response);
     };
 
-    debug("setting routes (restrpc): $base_url ", $lister);
     for my $call (keys %{ $dispatcher }) {
         my $endpoint = "$base_url/$call";
         post $endpoint, $handle_call;
@@ -165,12 +112,11 @@ register PLUGIN_NAME ,=> sub {
 };
 
 sub build_dispatcher_from_pod {
-    my ($pkgs, $endpoint) = @_;
+    my ($pkgs) = @_;
     debug("[build_dispatcher_from_pod]");
     return dispatch_table_from_pod(
-        plugin   => 'restrpc',
         packages => $pkgs,
-        endpoint => $endpoint,
+        label    => 'restrpc',
     );
 }
 
@@ -179,7 +125,7 @@ sub build_dispatcher_from_config {
     debug("[build_dispatcher_from_config] ");
 
     return dispatch_table_from_config(
-        plugin   => 'restrpc',
+        key      => 'restrpc',
         config   => $config,
         endpoint => $endpoint,
     );
@@ -357,12 +303,4 @@ Creates a (partial) dispatch table from data provided in POD.
 
 (c) MMXVII - Abe Timmerman <abeltje@cpan.org>
 
-=begin podcover_can_suck
-
-=head2 PLUGIN_NAME
-
-L<Test::Pod::Coverage> fails this test without this section :'(
-
-=end podcover_can_suck
-
 =cut
