@@ -3,22 +3,16 @@ use v5.10;
 use Dancer ':syntax';
 use Dancer::Plugin;
 use Scalar::Util 'blessed';
-use Time::HiRes 'time';
-
-our $VERSION = '1.09';
 
 no if $] >= 5.018, warnings => 'experimental::smartmatch';
 
-use constant PLUGIN_NAME => 'xmlrpc';
-
 use Dancer::RPCPlugin::CallbackResult;
 use Dancer::RPCPlugin::DispatchFromConfig;
 use Dancer::RPCPlugin::DispatchFromPod;
 use Dancer::RPCPlugin::DispatchItem;
 use Dancer::RPCPlugin::DispatchMethodList;
-use Dancer::RPCPlugin::ErrorResponse;
-use Dancer::RPCPlugin::FlattenData;
 
+use Params::Validate ':all';
 use RPC::XML::ParserFactory;
 
 my %dispatch_builder_map = (
@@ -26,100 +20,71 @@ my %dispatch_builder_map = (
     config => \&build_dispatcher_from_config,
 );
 
-register PLUGIN_NAME ,=> sub {
+register xmlrpc => sub {
     my($self, $endpoint, $arguments) = plugin_args(@_);
 
     my $publisher;
-    GIVEN: {
-        local $_ = $arguments->{publish} // 'config';
-        exists($dispatch_builder_map{$_}) && do {
+    given ($arguments->{publish} // 'config') {
+        when (exists $dispatch_builder_map{$_}) {
             $publisher = $dispatch_builder_map{$_};
+
             $arguments->{arguments} = plugin_setting() if $_ eq 'config';
-            last GIVEN;
-        };
-        do {
+        }
+        default {
             $publisher = $_;
         }
     }
+
+    my $code_wrapper = $arguments->{code_wrapper} // sub {
+        my $code = shift;
+        my $pkg  = shift;
+        $code->(@_);
+    };
+    my $callback = $arguments->{callback};
     my $dispatcher = $publisher->($arguments->{arguments}, $endpoint);
 
     my $lister = Dancer::RPCPlugin::DispatchMethodList->new();
     $lister->set_partial(
-        protocol => PLUGIN_NAME,
+        protocol => 'xmlrpc',
         endpoint => $endpoint,
         methods  => [ sort keys %{ $dispatcher } ],
     );
 
-    my $code_wrapper = $arguments->{code_wrapper}
-        ? $arguments->{code_wrapper}
-        : sub {
-            my $code = shift;
-            my $pkg  = shift;
-            $code->(@_);
-        };
-    my $callback = $arguments->{callback};
-
-    debug("Starting xmlrpc-handler build: ", $lister);
     my $handle_call = sub {
-        my ($ct) = (split /;\s*/, request->content_type, 2);
-        if ($ct !~ m{(text|application)/xml}) {
+        if (request->content_type ne 'text/xml') {
             pass();
         }
-        debug("[handle_xmlrpc_request] Processing: ", request->body);
+        debug("[handle_xmlrpc_call] Processing: ", request->body);
 
         local $RPC::XML::ENCODING = $RPC::XML::ENCODING ='UTF-8';
         my $p = RPC::XML::ParserFactory->new();
         my $request = $p->parse(request->body);
-        my $method_name = $request->name;
+        debug("[handle_xmlrpc_call] Parsed request: ", $request);
 
+        my $method_name = $request->name;
         if (! exists $dispatcher->{$method_name}) {
-            warning("$endpoint/#$method_name not found, pass()");
             pass();
         }
 
         content_type 'text/xml';
         my $response;
         my @method_args = map $_->value, @{$request->args};
-
-        debug("[handle_xmlrpc_call($method_name)] ", \@method_args);
-        my $start_request = time();
         my Dancer::RPCPlugin::CallbackResult $continue = eval {
-            local $Dancer::RPCPlugin::ROUTE_INFO = {
-                plugin        => PLUGIN_NAME,
-                endpoint      => $endpoint,
-                rpc_method    => $method_name,
-                full_path     => request->path,
-                http_method   => uc(request->method),
-            };
             $callback
                 ? $callback->(request(), $method_name, @method_args)
                 : callback_success();
         };
 
         if (my $error = $@) {
-            my $error_response = error_response(
-                error_code    => -32500,
-                error_message => $error,
-            );
-            status $error_response->return_status(PLUGIN_NAME);
-            $response = $error_response->as_xmlrpc_fault;
+            $response = {
+                faultCode => 500,
+                faultString => $error,
+            };
             return xmlrpc_response($response);
         }
-        if (!blessed($continue) || !$continue->isa('Dancer::RPCPlugin::CallbackResult')) {
-            my $error_response = error_response(
-                error_code    => -32500,
-                error_message => "Internal error: 'callback_result' wrong class " . blessed($continue),
-            );
-            status $error_response->return_status(PLUGIN_NAME);
-            $response = $error_response->as_xmlrpc_fault;
-        }
-        elsif (blessed($continue) && !$continue->success) {
-            my $error_response = error_response(
-                error_code    => $continue->error_code,
-                error_message => $continue->error_message,
-            );
-            status $error_response->return_status(PLUGIN_NAME);
-            $response = $error_response->as_xmlrpc_fault;
+        if (! $continue->success) {
+            $response->{faultCode} = $continue->error_code;
+            $response->{faultString} = $continue->error_message;
         }
         else {
             my Dancer::RPCPlugin::DispatchItem $di = $dispatcher->{$method_name};
@@ -130,33 +95,19 @@ register PLUGIN_NAME ,=> sub {
                 $code_wrapper->($handler, $package, $method_name, @method_args);
             };
 
-            debug("[handled_xmlrpc_request($method_name)] ", flatten_data($response));
             if (my $error = $@) {
-                my $error_response = blessed($error) && $error->can('as_xmlrpc_fault')
-                    ? $error
-                    : error_response(
-                            error_code    => -32500,
-                            error_message => $error,
-                            error_data    => $method_args[0],
-                    );
-                status $error_response->return_status(PLUGIN_NAME);
-                $response = $error_response->as_xmlrpc_fault;
+                $response = {
+                    faultCode => 500,
+                    faultString => $error,
+                };
             }
             if (blessed($response) && $response->can('as_xmlrpc_fault')) {
                 $response = $response->as_xmlrpc_fault;
             }
-            elsif (blessed($response)) {
-                $response = flatten_data($response);
-            }
         }
-        info( sprintf(
-            "[RPC::XMLRPC] request for %s took %.4fs",
-            $method_name, time() - $start_request
-        ));
         return xmlrpc_response($response);
     };
 
-    debug("setting route (xmlrpc): $endpoint ", $lister);
     post $endpoint, $handle_call;
 };
 
@@ -178,12 +129,11 @@ sub xmlrpc_response {
 }
 
 sub build_dispatcher_from_pod {
-    my ($pkgs, $endpoint) = @_;
+    my ($pkgs) = @_;
     debug("[build_dispatcher_from_pod]");
     return dispatch_table_from_pod(
-        plugin   => PLUGIN_NAME,
         packages => $pkgs,
-        endpoint => $endpoint,
+        label    => 'xmlrpc',
     );
 }
 
@@ -192,7 +142,7 @@ sub build_dispatcher_from_config {
     debug("[build_dispatcher_from_config] ");
 
     return dispatch_table_from_config(
-        plugin   => PLUGIN_NAME,
+        key      => 'xmlrpc',
         config   => $config,
         endpoint => $endpoint,
     );
@@ -369,12 +319,4 @@ Creates a (partial) dispatch table from data provided in POD.
 
 (c) MMXV - Abe Timmerman <abeltje@cpan.org>
 
-=begin podcover_can_suck
-
-=head2 PLUGIN_NAME
-
-L<Test::Pod::Coverage> fails this test without this section :'(
-
-=end podcover_can_suck
-
 =cut
